"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collector = void 0;
const index_1 = require("../crypto/index");
const Padder_1 = require("../Padder");
class Collector {
    constructor(map, numberOfSlices, verificationHash, hashEngine) {
        this.map = map;
        this.numberOfSlices = numberOfSlices;
        this.verificationHash = verificationHash;
        this.hashEngine = hashEngine;
        this.padder = new Padder_1.Padder();
    }
    /**
       * Test whether the passed data once hashed equals the verification hash
       *
       * @param data
       */
    check(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, index_1.hash)(data.toString(), this.hashEngine)
                .then(hashedData => hashedData === this.verificationHash);
        });
    }
    /**
       * @returns the concatenated slices into the obfuscated string
       */
    toObfuscated() {
        let o = '';
        for (let i = 0; i < this.numberOfSlices; i++) {
            const uncrumb = this.map.get(i);
            if (uncrumb === undefined) {
                throw new Error(`missing slice with index: ${i}`);
            }
            o += this.padder.unapply(uncrumb.toSlice());
        }
        o = this.padder.unapply(o);
        return Buffer.from(o);
    }
}
exports.Collector = Collector;
