"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Obfuscator = exports.DEFAULT_ROUNDS = exports.DEFAULT_KEY_STRING = exports.DEFAULT_HASH_ENGINE = void 0;
const feistel_cipher_1 = require("feistel-cipher");
exports.DEFAULT_HASH_ENGINE = feistel_cipher_1.SHA_256;
exports.DEFAULT_KEY_STRING = '8ed9dcc1701c064f0fd7ae235f15143f989920e0ee9658bb7882c8d7d5f05692'; // hash('crumbl by Edgewhere')
exports.DEFAULT_ROUNDS = 10;
class Obfuscator {
    constructor(cipher) {
        this.cipher = cipher;
    }
    /**
     * Obfuscate the passed data
     *
     * @param {string} data - the data to obfuscate
     * @returns the byte array of the obfuscated result
     */
    apply(data) {
        return (0, feistel_cipher_1.readable2Buffer)(this.cipher.encrypt(data));
    }
    /**
     * Deobfuscate the passed data
     *
     * @param {Buffer} obfuscated - the byte array to use
     * @returns the deobfuscated string
     */
    unapply(obfuscated) {
        return this.cipher.decrypt((0, feistel_cipher_1.toBase256Readable)(obfuscated));
    }
}
exports.Obfuscator = Obfuscator;
