export declare const MAX_SLICES = 4;
export declare const MAX_DELTA = 5;
export type Delta = 0 | 1 | 2 | 3 | 4 | 5;
export declare const MIN_INPUT_SIZE = 8;
export declare const MIN_SLICE_SIZE = 2;
export type Slice = string;
export interface Slicer {
    /**
     * Slice the passed data into an array of multiple parts.
     * The number of part is base on the properties `numberOfSlices` and `dataLength` this object was
     * initialized with.
     *
     * @param data the data to slice. It must be of the right length.
     * @returns Slices of the data, an array containing `numberOfSlices` element.
     *
     * @throws 'wrong number of slices' if we cannot build an array for this data. Most likely this Slicer isn't meant for this data.
     */
    readonly slice: (data: string) => [Slice, ...Array<Slice>];
    /**
     * Unslice the passed slices
     *
     * @param slices the slices to use
     * @returns the concatenated string
     */
    readonly unslice: (slices: [Slice, ...Array<Slice>]) => string;
}
/**
 * @param numberOfSlices Number of parts to splice data into
 * @param deltaMax The maximum difference of size between two slices.
 * It could be generated using `getDeltaMax` or be your specific choice.
 */
export declare const Slicer: (numberOfSlices: number, deltaMax: Delta) => Slicer;
/**
 * Find the maximum possible difference of size between two slices.
 *
 * @param dataLength the length of the source data
 * @param numberOfSlices the wanted number of slices
 *
 * @returns the maximum gap between the length of slices
 */
export declare const getDeltaMax: (dataLength: number, numberOfSlices: number) => Delta;
export * from './Seed';
//# sourceMappingURL=index.d.ts.map