"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDeltaMax = exports.Slicer = exports.MIN_SLICE_SIZE = exports.MIN_INPUT_SIZE = exports.MAX_DELTA = exports.MAX_SLICES = void 0;
const seedrandom_1 = __importDefault(require("seedrandom"));
const Seed_1 = require("./Seed");
const Padder_1 = require("../Padder");
exports.MAX_SLICES = 4; // The owner of the data + 3 trustees is optimal as of this version
exports.MAX_DELTA = 5;
exports.MIN_INPUT_SIZE = 8; // Input below 8 characters must be left-padded
exports.MIN_SLICE_SIZE = 2;
/**
 * @param numberOfSlices Number of parts to splice data into
 * @param deltaMax The maximum difference of size between two slices.
 * It could be generated using `getDeltaMax` or be your specific choice.
 */
const Slicer = (numberOfSlices, deltaMax) => {
    const padder = new Padder_1.Padder();
    if (numberOfSlices <= 0) {
        throw new Error(`number of slices too small: ${numberOfSlices}`);
    }
    return {
        slice: (data) => {
            if (data.length === 0)
                throw new Error('unsupported empty data');
            const fixedLength = Math.ceil(data.length / numberOfSlices) + deltaMax; // >= 1 because of ceil
            const slices = split(numberOfSlices, deltaMax, data).map(split => padder.apply(split, fixedLength));
            return slices;
        },
        unslice: (slices) => slices.map(padder.unapply).join('') // eslint-disable-line @typescript-eslint/unbound-method
    };
};
exports.Slicer = Slicer;
const split = (numberOfSlices, deltaMax, data) => buildSplitMask(numberOfSlices, deltaMax, data.length, (0, Seed_1.seedFor)(data))
    .map(mask => data.substring(mask.start, mask.end));
// TODO Enhance algorithm
const buildSplitMask = (numberOfSlices, deltaMax, dataLength, seed) => {
    const averageSliceLength = Math.floor(dataLength / numberOfSlices);
    const dm = Math.min(deltaMax, averageSliceLength - 1);
    const fullLength = dataLength;
    const masks = [];
    let catchUp = dataLength - averageSliceLength * numberOfSlices;
    let length = 0;
    let leftRound = numberOfSlices;
    const rng = (0, seedrandom_1.default)(seed);
    while (dataLength > 0) {
        const randomNum = rng() * dm / 2.0 + Math.floor(catchUp / leftRound);
        let addedNum = Math.min(dataLength, Math.ceil(randomNum) + averageSliceLength);
        // General rounding pb corrected at the end
        if (leftRound == 1 && length + addedNum < fullLength) {
            addedNum += fullLength - length - addedNum;
        }
        const m = {
            start: length,
            end: length + addedNum
        };
        masks.push(m);
        catchUp = fullLength - length - averageSliceLength * leftRound - (addedNum - averageSliceLength);
        leftRound--;
        length += addedNum;
        dataLength -= addedNum;
    }
    return masks;
};
/**
 * Find the maximum possible difference of size between two slices.
 *
 * @param dataLength the length of the source data
 * @param numberOfSlices the wanted number of slices
 *
 * @returns the maximum gap between the length of slices
 */
const getDeltaMax = (dataLength, numberOfSlices) => {
    if (numberOfSlices <= 0)
        throw new Error(`number of slices too small: ${numberOfSlices}`);
    const sliceSize = Math.floor(dataLength / numberOfSlices);
    if (dataLength <= exports.MIN_INPUT_SIZE || sliceSize <= exports.MIN_SLICE_SIZE) {
        return 0;
    }
    let deltaMax = 0;
    for (let delta = 1; delta < exports.MAX_DELTA + 1; delta++) {
        deltaMax = delta;
        if (delta < 2 * (sliceSize - exports.MIN_SLICE_SIZE)) {
            continue;
        }
        else {
            break;
        }
    }
    return deltaMax;
};
exports.getDeltaMax = getDeltaMax;
__exportStar(require("./Seed"), exports);
