"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Crumbl = exports.VERSION = void 0;
const feistel_cipher_1 = require("feistel-cipher");
const Dispatcher_1 = require("../Encrypter/Dispatcher");
const Encrypter_1 = require("../Encrypter");
const Hasher_1 = require("../Hasher");
const Obfuscator_1 = require("../Obfuscator");
const Slicer_1 = require("../Slicer");
const Padder_1 = require("../Padder");
exports.VERSION = '1'; // TODO Change when necessary (change of hash algorithm, modification of string structure, etc.)
class Crumbl {
    constructor(source, hashEngine, owners, trustees) {
        this.source = source;
        this.hashEngine = hashEngine;
        this.owners = owners;
        this.trustees = trustees;
        this.padder = new Padder_1.Padder();
        this.cipher = new feistel_cipher_1.FPECipher(Obfuscator_1.DEFAULT_HASH_ENGINE, Obfuscator_1.DEFAULT_KEY_STRING, Obfuscator_1.DEFAULT_ROUNDS);
    }
    process() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.doCrumbl();
        });
    }
    toHTML(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let crumbled = '';
            crumbled = yield this.doCrumbl();
            element.innerText = crumbled;
            return crumbled;
        });
    }
    doCrumbl() {
        return __awaiter(this, void 0, void 0, function* () {
            // 1- Obfuscate
            const obfuscator = new Obfuscator_1.Obfuscator(this.cipher);
            const obfuscated = obfuscator.apply(this.source);
            // 2- Pad
            const padded = this.padder.apply(obfuscated.toString(), obfuscated.length, true);
            // 3- Slice
            const numberOfSlices = 1 + Math.min(this.trustees.length, Slicer_1.MAX_SLICES);
            const deltaMax = (0, Slicer_1.getDeltaMax)(obfuscated.length, numberOfSlices);
            const slicer = (0, Slicer_1.Slicer)(numberOfSlices, deltaMax);
            const slices = slicer.slice(padded);
            // 4- Encrypt
            const crumbs = new Array();
            for (let i = 0; i < this.owners.length; i++) {
                const crumb = yield (0, Encrypter_1.encrypt)(slices[0], 0, this.owners[i]);
                crumbs.push(crumb);
            }
            const dispatcher = new Dispatcher_1.Dispatcher(numberOfSlices, this.trustees);
            const allocation = dispatcher.allocate();
            for (let i = 1; i < allocation.size + 1; i++) { // Initial index is 1 in allocation
                const maybeAllocated = allocation.get(i);
                if (maybeAllocated !== undefined) {
                    for (let j = 0; j < maybeAllocated.length; j++) {
                        const crumb = yield (0, Encrypter_1.encrypt)(slices[i], i, maybeAllocated[j]);
                        crumbs.push(crumb);
                    }
                }
            }
            // 5- Hash the source string
            const hasher = new Hasher_1.Hasher(crumbs);
            const hashered = yield hasher.apply(this.source);
            // 6- Finalize the output string
            const stringifiedCrumbs = new Array();
            for (let i = 0; i < crumbs.length; i++) {
                stringifiedCrumbs.push(crumbs[i].toString());
            }
            const crumbled = hashered + stringifiedCrumbs.join('') + '.' + exports.VERSION;
            return crumbled;
        });
    }
}
exports.Crumbl = Crumbl;
